window.Gleam.initSearch([{"doc":"crossbar","title":"crossbar","content":"# crossbar\n\n[![Package Version](https://img.shields.io/hexpm/v/crossbar)](https://hex.pm/packages/crossbar)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/crossbar/)\n\n## Usage\n\n```gleam\nimport gleam/io\nimport crossbar.{int, max_value, min_value, required, to_float, validate}\n\npub fn main() {\n  int(&quot;age&quot;, 16)\n  |&gt; to_float\n  |&gt; required\n  |&gt; min_value(18.0)\n  |&gt; max_value(21.0)\n  |&gt; validate\n  |&gt; io.debug\n}\n```\n\nThe snippet above will return the following error:\n\n```gleam\nError([FailedRule(&quot;age&quot;, &quot;min_value&quot;, &quot;must be at least 18.0&quot;)])\n```\n\n## Installation\n\n```sh\ngleam add crossbar\n```\n\nDocumentation can be found at &lt;https://hexdocs.pm/crossbar&gt;.\n","url":"index.html"},{"doc":"crossbar","title":"CrossBarError","content":"pub type CrossBarError {\n  FailedRule(name: String, rule: String, error: String)\n}\n The `CrossBarError` type is used to represent errors that occur during validation.\nFailedRule(name: String, rule: String, error: String)\n\n\nSynonyms:\ncrossbar.CrossBarError\ncrossbar CrossBarError","url":"crossbar.html#CrossBarError"},{"doc":"crossbar","title":"JsonMode","content":"pub type JsonMode {\n  Array\n  KeyValue\n}\n\nArray\n Return errors as an array of error strings\n json\n [&quot;is required&quot;, &quot;must be at least 5 characters&quot;]\n \n\nKeyValue\n Return errors as an object with the rule name as the key and the error string as the value\n json\n { &quot;required&quot;: &quot;is required&quot;, &quot;min_length&quot;: &quot;must be at least 5 characters&quot; }\n \n\nSynonyms:\ncrossbar.JsonMode\ncrossbar JsonMode","url":"crossbar.html#JsonMode"},{"doc":"crossbar","title":"ValidationResult","content":"pub type ValidationResult(a) =\n  Result(Field(a), List(CrossBarError))\n\n\nSynonyms:\ncrossbar.ValidationResult\ncrossbar ValidationResult","url":"crossbar.html#ValidationResult"},{"doc":"crossbar","title":"bool","content":"pub fn bool(name name: String, value value: Bool) -&gt; Field(Bool)\n Creates a new `Bool` field with the given name and value\nSynonyms:\ncrossbar.bool\ncrossbar bool","url":"crossbar.html#bool"},{"doc":"crossbar","title":"eq","content":"pub fn eq(\n  field field: Field(a),\n  name name: String,\n  value value: a,\n) -&gt; Field(a)\n The `eq` rule makes sure that the field is equal to the given value, strings are NOT compared securely, so this is NOT safe to use for passwords, all types are compared directly.\nSynonyms:\ncrossbar.eq\ncrossbar eq","url":"crossbar.html#eq"},{"doc":"crossbar","title":"extract_errors","content":"pub fn extract_errors(\n  result: Result(Field(a), List(CrossBarError)),\n) -&gt; List(#(String, String))\n Useful for extracting the errors as a list of tuples (#(rule_name, error_as_string)), this is useful for returning errors as JSON.\nSynonyms:\ncrossbar.extract_errors\ncrossbar extract_errors","url":"crossbar.html#extract_errors"},{"doc":"crossbar","title":"float","content":"pub fn float(\n  name name: String,\n  value value: Float,\n) -&gt; Field(Float)\n Creates a new `Float` field with the given name and value\nSynonyms:\ncrossbar.float\ncrossbar float","url":"crossbar.html#float"},{"doc":"crossbar","title":"has_errors","content":"pub fn has_errors(json_errors: List(#(String, Json))) -&gt; Bool\n Useful for checking if the result of `to_json_tuple` collected into a list has any errors, this is useful for checking if any of the fields failed validation.\n\n ## Example\n\n \n let first_name =\n   string(&quot;first_name&quot;, &quot;John&quot;)\n   |&gt; required\n   |&gt; min_length(3)\n   |&gt; to_json_tuple(KeyValue)\n\n let last_name =\n   string(&quot;last_name&quot;, &quot;Smith&quot;)\n   |&gt; required\n   |&gt; min_length(1)\n   |&gt; max_length(10)\n   |&gt; to_json_tuple(KeyValue)\n\n let errors = [first_name, last_name]\n\n case has_errors(errors) {\n   True -&gt; io.println(&quot;There are errors&quot;)\n   False -&gt; io.println(&quot;There are no errors&quot;)\n }\n \nSynonyms:\ncrossbar.has_errors\ncrossbar has_errors","url":"crossbar.html#has_errors"},{"doc":"crossbar","title":"int","content":"pub fn int(name name: String, value value: Int) -&gt; Field(Int)\n Creates a new `Int` field with the given name and value\nSynonyms:\ncrossbar.int\ncrossbar int","url":"crossbar.html#int"},{"doc":"crossbar","title":"max_length","content":"pub fn max_length(\n  field field: Field(String),\n  length length: Int,\n) -&gt; Field(String)\n The `max_length` rule makes sure that the field is at most the given length, this is the expected behaviour in the following cases:\n &gt; `Int`: the length of the string representation of the number, this isn&#39;t very useful to you, but it&#39;s here for completeness sake. You probably want to use `max_value` instead.\n\n &gt; `Float`: the length of the string representation of the number, this also isn&#39;t very useful to you, but it&#39;s here for completeness sake. You probably want to use `max_value` instead.\n\n &gt; `String`: the length of the string is evaluated directly\n\n &gt; `Bool`: this also isn&#39;t very useful to you, but it&#39;s here for completeness sake.\n\n NOTE: This function has been momentarily restricted to `String` fields, because it&#39;s not very useful for other types, open an issue if you ever find a use for it.\nSynonyms:\ncrossbar.max_length\ncrossbar max_length","url":"crossbar.html#max_length"},{"doc":"crossbar","title":"max_value","content":"pub fn max_value(\n  field field: Field(Float),\n  size size: Float,\n) -&gt; Field(Float)\n The `max_value` rule makes sure that the field is at most the given (byte) size.\n &gt; Strings are counted in bytes (as bit arrays), `Int`s and `Float`s are evaluated directly, `Bool`s are treated as their binary equivalent (0 or 1).\n\n NOTE: This function has been momentarily restricted to `Float` fields, because it&#39;s not very useful for other types, open an issue if you ever find a use for it. There is also a `to_float` function to transform int fields to float fields (meant to be used before you add any rules)\nSynonyms:\ncrossbar.max_value\ncrossbar max_value","url":"crossbar.html#max_value"},{"doc":"crossbar","title":"min_length","content":"pub fn min_length(\n  field field: Field(String),\n  length length: Int,\n) -&gt; Field(String)\n The `min_length` rule makes sure that the field is at least the given length, this is the expected behaviour in the following cases:\n &gt; `Int`: the length of the string representation of the number, this isn&#39;t very useful to you, but it&#39;s here for completeness sake. You probably want to use `min_value` instead.\n\n &gt; `Float`: the length of the string representation of the number, this isn&#39;t very useful to you, but it&#39;s here for completeness sake. You probably want to use `min_value` instead.\n\n &gt; `String`: the length of the string is evaluated directly\n\n &gt; `Bool`: this also isn&#39;t very useful to you, but it&#39;s here for completeness sake.\n\n NOTE: This function has been momentarily restricted to `String` fields, because it&#39;s not very useful for other types, open an issue if you ever find a use for it.\nSynonyms:\ncrossbar.min_length\ncrossbar min_length","url":"crossbar.html#min_length"},{"doc":"crossbar","title":"min_value","content":"pub fn min_value(\n  field field: Field(Float),\n  size size: Float,\n) -&gt; Field(Float)\n The `min_value` rule makes sure that the field is at least the given (byte where it applies) size.\n &gt; Strings are counted in bytes (as bit arrays), `Int`s and `Float`s are evaluated directly, `Bool`s are treated as their binary equivalent (0 or 1).\n\n NOTE: This function has been momentarily restricted to `Float` fields, because it&#39;s not very useful for other types, open an issue if you ever find a use for it. There is also a `to_float` function to transform int fields to float fields (meant to be used before you add any rules)\nSynonyms:\ncrossbar.min_value\ncrossbar min_value","url":"crossbar.html#min_value"},{"doc":"crossbar","title":"not_eq","content":"pub fn not_eq(\n  field field: Field(a),\n  name name: String,\n  value value: a,\n) -&gt; Field(a)\n The `not_eq` rule makes sure that the field is not equal to the given value, strings are NOT compared securely, so this is NOT safe to use for passwords, other types are compared directly.\nSynonyms:\ncrossbar.not_eq\ncrossbar not_eq","url":"crossbar.html#not_eq"},{"doc":"crossbar","title":"regex","content":"pub fn regex(\n  field field: Field(a),\n  rule_name name: String,\n  regex regex: Regex,\n  error_message error: String,\n) -&gt; Field(a)\n The `regex` rule makes sure that the field matches the given regex, you are required to compile the regex yourself, if you want to use an uncompiled regex, use the `uncompiled_regex` rule instead.\n\n ## Example\n\n \n import gleam/regex\n\n let options = Options(case_insensitive: False, multi_line: True)\n let assert Ok(re) = compile(&quot;^[0-9]&quot;, with: options)\n\n string(&quot;name&quot;, &quot;1john&quot;)\n |&gt; regex(&quot;starts_with_number&quot;, re,  &quot;must start with a number&quot;)\n \nSynonyms:\ncrossbar.regex\ncrossbar regex","url":"crossbar.html#regex"},{"doc":"crossbar","title":"required","content":"pub fn required(field field: Field(a)) -&gt; Field(a)\n The required rule makes sure that the field is not empty, this is the expected behaviour in the following cases:\n &gt; `Int`: **0** is considered empty\n\n &gt; `Float`: **0.0** is also considered empty\n\n &gt; `String`: &quot;&quot; (or anything that trims down to that) is considered empty\n\n &gt; `Bool`: this isn&#39;t really a thing, but it&#39;s here for completeness sake and it will always return true, because a bool is never empty (unless it is wrapped in an option)\n\nSynonyms:\ncrossbar.required\ncrossbar required","url":"crossbar.html#required"},{"doc":"crossbar","title":"rule_to_error_string","content":"pub fn rule_to_error_string(rule: Rule(a)) -&gt; String\n Get the default error message for a rule - this is internally used for error states\nSynonyms:\ncrossbar.rule_to_error_string\ncrossbar rule_to_error_string","url":"crossbar.html#rule_to_error_string"},{"doc":"crossbar","title":"rule_to_string","content":"pub fn rule_to_string(rule: Rule(a)) -&gt; String\n Returns the string representation of a rule - this is internally used for error states\nSynonyms:\ncrossbar.rule_to_string\ncrossbar rule_to_string","url":"crossbar.html#rule_to_string"},{"doc":"crossbar","title":"string","content":"pub fn string(\n  name name: String,\n  value value: String,\n) -&gt; Field(String)\n Creates a new `String` field with the given name and value\nSynonyms:\ncrossbar.string\ncrossbar string","url":"crossbar.html#string"},{"doc":"crossbar","title":"to_float","content":"pub fn to_float(field: Field(Int)) -&gt; Field(Float)\n Convenient function to convert an `Int` to a `Float`, you should use this BEFORE applying any rules.\n\n ## Example\n\n \n int(&quot;age&quot;, 6)\n |&gt; to_float\n |&gt; min_value(5.0)\n \n\nSynonyms:\ncrossbar.to_float\ncrossbar to_float","url":"crossbar.html#to_float"},{"doc":"crossbar","title":"to_json_tuple","content":"pub fn to_json_tuple(\n  field: Field(a),\n  mode: JsonMode,\n) -&gt; #(String, Json)\n Transform a field into a tuple that can be used to generate JSON, this is useful for returning errors as JSON.\n\n ## Example\n\n \n let first_name =\n   string(&quot;first_name&quot;, &quot;&quot;)\n   |&gt; required\n   |&gt; min_length(3)\n   |&gt; to_json_tuple(KeyValue)\n\n let last_name =\n   string(&quot;last_name&quot;, &quot;Smith&quot;)\n   |&gt; required\n   |&gt; min_length(1)\n   |&gt; max_length(3)\n   |&gt; to_json_tuple(KeyValue)\n\n json.object([first_name, last_name])\n |&gt; json.to_string\n |&gt; io.println\n \n\n The above example will produce the following JSON:\n\n json\n {\n   &quot;first_name&quot;: {\n     &quot;required&quot;: &quot;is required&quot;,\n     &quot;min_length&quot;: &quot;must be at least 3 characters&quot;\n   },\n   &quot;last_name&quot;: {\n     &quot;max_length&quot;: &quot;must not be longer than 3 characters&quot;\n   }\n }\n \nSynonyms:\ncrossbar.to_json_tuple\ncrossbar to_json_tuple","url":"crossbar.html#to_json_tuple"},{"doc":"crossbar","title":"validate","content":"pub fn validate(\n  field: Field(a),\n) -&gt; Result(Field(a), List(CrossBarError))\n Run the validation on the given field, returns an `Ok` if the validation was successful, otherwise returns an `Error` with a list of errors.\n\n ## Example\n\n \n let field = int(&quot;age&quot;, 6)\n |&gt; to_float\n |&gt; min_value(5.0)\n |&gt; validate\n \nSynonyms:\ncrossbar.validate\ncrossbar validate","url":"crossbar.html#validate"},{"doc":"crossbar","title":"validate_many","content":"pub fn validate_many(\n  fields: List(Field(a)),\n) -&gt; List(#(String, List(CrossBarError)))\n Validate a list of fields, returns a list of tuples with the field name and a list of errors - unfortunately, currently, only supports validating fields of the same type.\nSynonyms:\ncrossbar.validate_many\ncrossbar validate_many","url":"crossbar.html#validate_many"},{"doc":"crossbar","title":"with_validator","content":"pub fn with_validator(\n  field field: Field(a),\n  rule_name name: String,\n  validator func: fn(a) -&gt; Bool,\n  error error: String,\n) -&gt; Field(a)\n The `with_validator` rule makes sure that the field passes the given validator function, the function should return a `Bool` and take the field value as its only argument.\n\n ## Example\n\n \n let validator = fn(x) { x &gt; 5 }\n let error = &quot;must be greater than 5&quot;\n\n int(&quot;age&quot;, 6)\n |&gt; with_validator(&quot;greater_than_5&quot;, validator, error)\n \nSynonyms:\ncrossbar.with_validator\ncrossbar with_validator","url":"crossbar.html#with_validator"},{"doc":"crossbar","title":"crossbar","content":"","url":"crossbar.html"}]);